package net.blumia.pcmdroid.model

import android.net.Uri
import android.util.Log
import okhttp3.HttpUrl
import okhttp3.HttpUrl.Companion.toHttpUrlOrNull
import org.json.JSONArray
import org.json.JSONException
import org.json.JSONObject
import java.net.URLDecoder
import kotlin.io.path.Path

data class Song (
    val filePath: String, // with file name
    val displayName: String = "",
    val modifiedTime: Long = 0,
    val fileSize: Long = 0,
    val additionalInfo: Boolean = false, // if sidecar meta-info json file exist.
    val url: String,
) {
    fun displayName(): String {
        return if (displayName.isNotEmpty()) displayName else Uri.parse(filePath).lastPathSegment ?: "???"
    }
}

data class Folder (
    val folderPath: String,
) {
    fun displayName(): String {
        return URLDecoder.decode(Uri.parse(folderPath).lastPathSegment ?: "???", "UTF-8")
    }
}

fun parseFromJsonString(mediaRootUrl: String, baseFolder: String, jsonString: String): Pair<List<Folder>, List<Song>> {
    val folders = mutableListOf<Folder>()
    val songs = mutableListOf<Song>()

    val mediaRoot = if (mediaRootUrl.endsWith('/')) mediaRootUrl else "$mediaRootUrl/"
    val filePath = if (baseFolder.endsWith('/') || baseFolder == "") baseFolder else "$baseFolder/"

    try {
        val jsonObj = JSONObject(jsonString)
        val result = jsonObj.getJSONObject("result")
        val data = result.getJSONObject("data")
        val subFolders = data.getJSONArray("subFolderList")
        val musicList = if (data.has("musicList")) data.getJSONArray("musicList") else JSONArray()
        for (i in 0 until subFolders.length()) {
            val rawFolderName = subFolders.getString(i)
            folders.add(Folder(rawFolderName))
        }
        for (i in 0 until musicList.length()) {
            val fileObject = musicList.getJSONObject(i)
            val rawFileName = fileObject.getString("fileName")
            val displayName = if (fileObject.has("displayName")) fileObject.getString("displayName") else ""
            val filePathWithName = filePath + rawFileName
            var fileUrl = if (fileObject.has("url")) fileObject.getString("url") else (mediaRoot + filePathWithName)
            val fileName = URLDecoder.decode(rawFileName, "UTF-8")
            val modifiedTime = fileObject.getLong("modifiedTime")
            val fileSize = fileObject.getLong("fileSize")

            // it's possible that server return a url start without protocol.
            if (fileUrl.startsWith("//")) {
                fileUrl = mediaRootUrl.toHttpUrlOrNull()?.scheme + fileUrl
            }

            songs.add(
                Song(
                    filePathWithName,
                    displayName = if (displayName.isNotEmpty()) displayName else fileName,
                    modifiedTime = modifiedTime,
                    url = fileUrl,
                    fileSize = fileSize,
                )
            )
        }
    } catch (e: JSONException) {
        e.printStackTrace()
        Log.d("vvv", jsonString)
    }

    return Pair(folders, songs)
}