package net.blumia.pcmdroid.service

import android.app.PendingIntent.FLAG_IMMUTABLE
import android.app.PendingIntent.FLAG_UPDATE_CURRENT
import android.content.Intent
import android.os.Build
import android.util.Log
import androidx.core.app.TaskStackBuilder
import androidx.media3.common.AudioAttributes
import androidx.media3.common.MediaItem
import androidx.media3.common.MediaMetadata
import androidx.media3.common.MediaMetadata.FOLDER_TYPE_MIXED
import androidx.media3.common.util.UnstableApi
import androidx.media3.exoplayer.ExoPlayer
import androidx.media3.session.LibraryResult
import androidx.media3.session.MediaLibraryService
import androidx.media3.session.MediaSession
import com.google.common.util.concurrent.Futures
import com.google.common.util.concurrent.ListenableFuture
import net.blumia.pcmdroid.MainActivity

class PlaybackService : MediaLibraryService() {

    private lateinit var player: ExoPlayer
    private lateinit var mediaLibrarySession: MediaLibrarySession
    private val librarySessionCallback = CustomMediaLibrarySessionCallback()

    private inner class CustomMediaLibrarySessionCallback
        : MediaLibrarySession.Callback
    {
        @androidx.annotation.OptIn(UnstableApi::class)
        override fun onGetLibraryRoot(
            session: MediaLibrarySession,
            browser: MediaSession.ControllerInfo,
            params: LibraryParams?
        ): ListenableFuture<LibraryResult<MediaItem>> {
            Log.d("aaa", "onGetLibraryRoot ${params.toString()}")
            val metadata = MediaMetadata.Builder()
                .setTitle("Root")
                .setIsPlayable(false)
                .setFolderType(FOLDER_TYPE_MIXED)
                .build()
            val item = MediaItem.Builder()
                .setMediaId("test")
                .setMediaMetadata(metadata)
                .build()
            return Futures.immediateFuture(LibraryResult.ofItem(item, params))
            //MediaMetadata
//            return super.onGetLibraryRoot(session, browser, params)
        }

        override fun onGetItem(
            session: MediaLibrarySession,
            browser: MediaSession.ControllerInfo,
            mediaId: String
        ): ListenableFuture<LibraryResult<MediaItem>> {
            Log.d("aaa", "onGetItem $mediaId")
            return super.onGetItem(session, browser, mediaId)
        }
    }

    override fun onGetSession(controllerInfo: MediaSession.ControllerInfo): MediaLibrarySession {
        return mediaLibrarySession
    }

    override fun onCreate() {
        super.onCreate()
        initializeSessionAndPlayer()
    }

    override fun onDestroy() {
        player.release()
        mediaLibrarySession.release()
        super.onDestroy()
    }

    @androidx.annotation.OptIn(UnstableApi::class)
    private fun initializeSessionAndPlayer() {
        player =
            ExoPlayer.Builder(this)
                .setAudioAttributes(AudioAttributes.DEFAULT, true)
                .build()

        val mainActivityIntent = Intent(this, MainActivity::class.java)
        val pendingIntent =
            TaskStackBuilder.create(this).run {
                addNextIntent(mainActivityIntent)

                val immutableFlag = if (Build.VERSION.SDK_INT >= 23) FLAG_IMMUTABLE else 0
                getPendingIntent(0, immutableFlag or FLAG_UPDATE_CURRENT)
            }

        mediaLibrarySession =
            MediaLibrarySession.Builder(this, player, librarySessionCallback)
                .setSessionActivity(pendingIntent!!)
                .build()
    }
}