package net.blumia.pcmdroid.ui.screen.editserver

import androidx.compose.foundation.layout.*
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.AddCircle
import androidx.compose.material.icons.filled.Close
import androidx.compose.material.icons.filled.Edit
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import net.blumia.pcmdroid.model.Server

@Composable
fun EditServerScreen(
    server: Server,
    onCloseActionTriggered: () -> Unit = {},
    onSaveButtonTriggered: (Server) -> Unit = {},
)
{
    Scaffold(
        topBar = {
            TopAppBar(
                backgroundColor = Color.Transparent,
                elevation = 0.dp,
                title = {},
                navigationIcon = {
                    IconButton(onClick = onCloseActionTriggered) {
                        Icon(imageVector = Icons.Filled.Close, contentDescription = "Close")
                    }
                },
            )
        }
    ) {
        Column(
            modifier = Modifier
                .fillMaxSize()
                .verticalScroll(rememberScrollState())
                .padding(20.dp),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            Icon(
                modifier = Modifier.size(50.dp),
                imageVector = Icons.Filled.Edit,
                contentDescription = null,
            )
            Text(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(vertical = 10.dp),
                text = "Edit Server",
                textAlign = TextAlign.Center,
                style = MaterialTheme.typography.h5,
            )
            Text(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(bottom = 10.dp),
                text = "Edit existed Private Cloud Music server",
                textAlign = TextAlign.Center,
                style = MaterialTheme.typography.subtitle2,
            )

            var shortNameStr by rememberSaveable { mutableStateOf(server.shortName) }
            var nameStr by rememberSaveable { mutableStateOf(server.name) }
            var urlStr by rememberSaveable { mutableStateOf(server.url) }
            var baseFolderNameStr by rememberSaveable { mutableStateOf(server.baseFolderName) }
            var mediaBaseUrlStr by rememberSaveable { mutableStateOf(server.mediaBaseUrl) }
            var preferredFormatsStr by rememberSaveable { mutableStateOf(server.preferredFormats) }

            OutlinedTextField(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(vertical = 10.dp),
                singleLine = true,
                value = shortNameStr,
                onValueChange = { value -> shortNameStr = value},
                label = { Text("Server Short Name") },
            )

            OutlinedTextField(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(vertical = 10.dp),
                singleLine = true,
                value = nameStr,
                onValueChange = { value -> nameStr = value},
                label = { Text("Server Name") },
            )

            OutlinedTextField(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(vertical = 10.dp),
                keyboardOptions = KeyboardOptions(
                    keyboardType = KeyboardType.Uri,
                ),
                singleLine = true,
                value = urlStr,
                onValueChange = { value -> urlStr = value},
                label = { Text("Server API Url") },
            )

            OutlinedTextField(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(vertical = 10.dp),
                singleLine = true,
                value = baseFolderNameStr,
                onValueChange = { value -> baseFolderNameStr = value},
                label = { Text("Base folder name") },
            )

            OutlinedTextField(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(vertical = 10.dp),
                keyboardOptions = KeyboardOptions(
                    keyboardType = KeyboardType.Uri,
                ),
                singleLine = true,
                value = mediaBaseUrlStr,
                onValueChange = { value -> mediaBaseUrlStr = value},
                label = { Text("Media base url") },
            )

            OutlinedTextField(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(vertical = 10.dp),
                value = preferredFormatsStr,
                onValueChange = { value -> preferredFormatsStr = value},
                label = { Text("Preferred formats") },
            )

            Button(
                modifier = Modifier.fillMaxWidth(),
                onClick = {
                    onSaveButtonTriggered(
                        Server(
                            serverId = server.serverId,
                            name = nameStr,
                            shortName = shortNameStr,
                            url = urlStr,
                            baseFolderName = baseFolderNameStr,
                            mediaBaseUrl = mediaBaseUrlStr,
                            preferredFormats = preferredFormatsStr,
                        )
                    )
                },
            ) {
                Text("Save")
            }
        }
    }
}

@Preview(showBackground = true)
@Composable
fun EditServerScreenPreview()
{
    EditServerScreen(
        Server(
            -1,
            "http://localhost/api.cgi",
            "Demo server",
            "Demo",
            "http://localhost/",
            "ogg,mp3",
            "demo"
        )
    )
}