package net.blumia.pcmdroid.ui.screen.main

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.*
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.vector.ImageVector
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import net.blumia.pcmdroid.R
import net.blumia.pcmdroid.model.Server

@OptIn(ExperimentalMaterialApi::class)
@Composable
fun RowScope.ActionIcon(
    imageVector: ImageVector = Icons.Filled.Share,
    contentDescription: String = "Share",
    onActionTriggered: () -> Unit = {},
) {
    Surface(
        modifier = Modifier.weight(1f),
        onClick = onActionTriggered,
    ) {
        Column(
            modifier = Modifier
                .fillMaxWidth()
                .padding(10.dp),
            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            Icon(imageVector = imageVector, contentDescription = contentDescription)
            Text(contentDescription)
        }
    }
}

@OptIn(ExperimentalMaterialApi::class)
@Composable
fun ServerSummaryDrawer(
    server: Server,
    onShareActionTriggered: (Server) -> Unit = {},
    onEditActionTriggered: (Server) -> Unit = {},
    onDeleteActionTriggered: (Server) -> Unit = {},
) {
    Column(
        modifier = Modifier.padding(vertical = 16.dp)
    ) {
        Column(
            modifier = Modifier.padding(horizontal = 16.dp)
        ) {
            Icon(
                modifier = Modifier.size(50.dp),
                imageVector = Icons.Filled.LibraryMusic,
                contentDescription = null,
            )
            Text(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(vertical = 10.dp),
                text = server.displayName(),
    //            textAlign = TextAlign.Center,
                style = MaterialTheme.typography.h5,
            )
        }
//        Row(
//            modifier = Modifier.fillMaxWidth(),
//        ) {
//            ActionIcon(
//                imageVector = Icons.Filled.Share,
//                contentDescription = "Share",
//            )
//            ActionIcon(
//                imageVector = Icons.Filled.Edit,
//                contentDescription = "Edit",
//            )
//        }
        ListItem(
            modifier = Modifier.clickable {
                onShareActionTriggered(server)
            },
            icon = {
                Icon(Icons.Filled.Share, null)
            },
            text = {
                Text(stringResource(id = R.string.share_this_server))
            },
        )
        ListItem(
            modifier = Modifier.clickable {
                onEditActionTriggered(server)
            },
            icon = {
                Icon(Icons.Filled.Edit, null)
            },
            text = {
                Text(stringResource(id = R.string.edit_server_info))
            },
        )
        ListItem(
            modifier = Modifier.clickable {
                onDeleteActionTriggered(server)
            },
            icon = {
                Icon(Icons.Filled.Delete, null)
            },
            text = {
                Text(stringResource(id = R.string.delete_server))
            },
        )
    }
}

@Preview(showBackground = true)
@Composable
fun ServerSummaryDrawerPreview()
{
    ServerSummaryDrawer(
        Server(
            -1,
            "http://localhost/",
            "Demo server",
            "Demo",
            "http://localhost/",
            "ogg,mp3",
            "pcm"
        )
    )
}