package net.blumia.pcmdroid.ui.screen.settings

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Build
import android.provider.DocumentsContract
import androidx.annotation.RequiresApi
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.ArrowBack
import androidx.compose.material.icons.filled.Close
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp

fun openDirectory(context: Context) {
    // Choose a directory using the system's file picker.
    val intent = Intent(Intent.ACTION_OPEN_DOCUMENT_TREE).apply {
        // Provide read access to files and sub-directories in the user-selected
        // directory.
        flags = Intent.FLAG_GRANT_READ_URI_PERMISSION

        // Optionally, specify a URI for the directory that should be opened in
        // the system file picker when it loads.
        // , pickerInitialUri: Uri
//        putExtra(DocumentsContract.EXTRA_INITIAL_URI, pickerInitialUri)
    }
    context as Activity

    context.startActivityForResult(intent, 1919810)
}

@OptIn(ExperimentalMaterialApi::class)
@Preview(showBackground = true)
@Composable
fun SettingsScreen(
    onBackActionTriggered: () -> Unit = {},
) {
    Scaffold(
        topBar = {
            TopAppBar(
                backgroundColor = Color.Transparent,
                elevation = 0.dp,
                title = { Text("Settings") },
                navigationIcon = {
                    IconButton(onClick = onBackActionTriggered) {
                        Icon(imageVector = Icons.Filled.ArrowBack, contentDescription = "Close")
                    }
                },
            )
        }
    ) {
        Column(
            modifier = Modifier.fillMaxSize()
        ) {
            ListItem(
                text = {
                    Text(
                        text = "Local storage",
                        style = MaterialTheme.typography.overline,
                        color = MaterialTheme.colors.primary,
                    )
                }
            )

            // cannot do the cast here or it will break the preview.
            val currentActivity = LocalContext.current
            ListItem(
                modifier = Modifier.clickable { ->
                    openDirectory(currentActivity)
                },
                text = { Text("Location") },
                secondaryText = { Text("Click to select storage location.") },
            )
        }
    }
}